import {Component} from '@angular/core'
import {Observable} from 'rxjs'
import {GroupService} from '../../services/group.service'
import {EmployeeGroup, EmployeePermission} from '../../../shared/model/employee'
import {takeUntil} from 'rxjs/operators'
import {animate, state, style, transition, trigger} from '@angular/animations'
import {AccountService} from '../../../accounts/services/account.service'
import {ErrorHandlingComponent} from '../../../shared/components/subscribing.component'
import {ActivatedRoute, Router} from '@angular/router'
import {ErrorModel, ErrorService} from '../../../shared/service/error.service'
import {AlertService} from '../../../shared/service/alert.service'

@Component({
  selector: 'cre-groups',
  templateUrl: './list.component.html',
  styleUrls: ['./list.component.sass'],
  animations: [
    trigger('detailExpand', [
      state('collapsed', style({height: '0px', minHeight: '0'})),
      state('expanded', style({height: '*'})),
      transition('expanded <=> collapsed', animate('225ms cubic-bezier(0.4, 0.0, 0.2, 1)'))
    ])
  ]
})
export class ListComponent extends ErrorHandlingComponent {
  groups$: Observable<EmployeeGroup[]>
  defaultGroup: EmployeeGroup = null
  columns = ['name', 'permissionCount', 'employeeCount', 'defaultGroup', 'editGroup']
  expandedElement: EmployeeGroup | null

  handledErrorModels: ErrorModel[] = [{
    filter: error => error.status === 404,
    consumer: () => this.alertService.pushWarning('Aucun groupe par défaut n\'a été défini sur cet ordinateur')
  }]

  constructor(
    private groupService: GroupService,
    private accountService: AccountService,
    private alertService: AlertService,
    errorService: ErrorService,
    router: Router,
    activatedRoute: ActivatedRoute
  ) {
    super(errorService, activatedRoute, router)
  }

  ngOnInit(): void {
    this.groups$ = this.groupService.all.pipe(takeUntil(this.destroy$))
    this.subscribe(
      this.groupService.defaultGroup,
      group => this.defaultGroup = group,
    )
  }

  setDefaultGroup(group: EmployeeGroup) {
    this.subscribe(
      this.groupService.setDefaultGroup(group),
      () => this.defaultGroup = group
    )
  }

  isDefaultGroup(group: EmployeeGroup): boolean {
    return this.defaultGroup && this.defaultGroup.id == group.id
  }

  get canViewEmployee(): boolean {
    return this.accountService.hasPermission(EmployeePermission.VIEW_EMPLOYEE)
  }

  get canEditGroup(): boolean {
    return this.accountService.hasPermission(EmployeePermission.EDIT_EMPLOYEE_GROUP)
  }

  get canSetBrowserDefaultGroup(): boolean {
    return this.accountService.hasPermission(EmployeePermission.SET_BROWSER_DEFAULT_GROUP)
  }
}
